package at.bestsolution.rsd.rt.rest.apachehttp;

import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import org.apache.http.client.fluent.Request;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;

import at.bestsolution.rsd.rt.FileStream;

public class ApachePostMultipartRequest extends BaseApacheRequest {
	private Map<String, FileStream> fileStreams = new HashMap<>();
	
	public ApachePostMultipartRequest(HttpExecutor httpExecutor, String endpoint) {
		super(httpExecutor, endpoint);
	}
	
	@Override
	public void addParameter(String name, Object value) {
		if( value instanceof FileStream ) {
			fileStreams.put(name, (FileStream) value);
		} else {
			super.addParameter(name, value);	
		}
	}
	
	@Override
	protected Request createRequest() throws URISyntaxException {
		URIBuilder builder = new URIBuilder(endpoint);
		
		MultipartEntityBuilder bodyBuilder = MultipartEntityBuilder.create()
		        .setCharset(StandardCharsets.UTF_8)
		        .setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
		
		for( Entry<String, FileStream> e : fileStreams.entrySet() ) {
			bodyBuilder.addBinaryBody(e.getKey(), e.getValue().getStream(), ContentType.DEFAULT_BINARY, e.getValue().filename());	
		}
		
		for( Consumer<BiConsumer<String,String>> p : parameters ) {
			p.accept(bodyBuilder::addTextBody);
		}
		
		return org.apache.http.client.fluent.Request.Post(builder.build()).body(bodyBuilder.build());
	}
}
