package at.bestsolution.rsd.rt.rest.apachehttp;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Map;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import org.apache.http.HttpEntity;

import at.bestsolution.rsd.rt.rest.ResponseStream;

public class ApacheResponseStream implements ResponseStream {
	private final HttpEntity entity;
	private Map<String, String> header;
	private final Supplier<Map<String, String>> headerSupplier;
	
	public ApacheResponseStream(HttpEntity entity, Supplier<Map<String, String>> headerSupplier) {
		this.entity = entity;
		this.headerSupplier = headerSupplier;
	}
	
	@Override
	public Map<String, String> headers() {
		if( header == null ) {
			header = Collections.unmodifiableMap(headerSupplier.get().entrySet().stream().collect(Collectors.toMap(e -> e.getKey().toLowerCase(), e -> e.getValue())));
		}
		return header;
	}
	
	@Override
	public InputStream getContent() {
		try {
			return entity.getContent();
		} catch (UnsupportedOperationException | IOException e) {
			throw new IllegalStateException(e);
		}
	}

}
