package at.bestsolution.rsd.rt.rest.apachehttp;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.fluent.Response;

import at.bestsolution.rsd.rt.rest.Request;
import at.bestsolution.rsd.rt.rest.ResponseStream;

public abstract class BaseApacheRequest implements Request {
	protected final String endpoint;
	protected final List<Consumer<BiConsumer<String,String>>> parameters = new ArrayList<>();
	private Map<String,String> headerMap = new HashMap<>();
	private final HttpExecutor httpExecutor;
	
	public BaseApacheRequest(HttpExecutor httpExecutor, String endpoint) {
		this.endpoint = endpoint;
		this.httpExecutor = httpExecutor;
	}

	@Override
	public void addCookie(String name, Object value) {
		// TODO Auto-generated method stub

	}

	@Override
	public void setHeader(String name, String value) {
		headerMap.put(name, value);
	}
	
	@Override
	public void addParameter(String name, Object value) {
		if( value != null ) {
			parameters.add( keyValue -> keyValue.accept(name, Objects.toString(value)) );
		}
	}
	
	@Override
	public void addParameter(String name, Consumer<OutputStream> writer) {
		parameters.add( keyValue -> {
			try( ByteArrayOutputStream stream = new ByteArrayOutputStream() ) {
				writer.accept(stream);
				stream.flush();
				keyValue.accept(name, new String(stream.toByteArray()));
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}
		});
	}

	@Override
	public CompletableFuture<ResponseStream> call() {
		return CompletableFuture.supplyAsync(this::invokeRequest);
	}
	
	@Override
	public CompletableFuture<ResponseStream> call(Executor executor) {
		return CompletableFuture.supplyAsync(this::invokeRequest,executor);
	}
	
	@Override
	public ResponseStream execute() {
		return invokeRequest();
	}
	
	private ResponseStream invokeRequest() {
		try {
			org.apache.http.client.fluent.Request request = createRequest();
			for( Entry<String, String> e : headerMap.entrySet() ) {
				request.addHeader(e.getKey(), e.getValue());
			}
			Response resp = httpExecutor.execute(request);
			
			HttpResponse httpResponse = resp.returnResponse();

			if (httpResponse.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				throw new IllegalStateException(httpResponse.getStatusLine().getReasonPhrase());
			}
			
			return new ApacheResponseStream(httpResponse.getEntity(), () -> Stream.of(httpResponse.getAllHeaders()).collect(Collectors.toMap(Header::getName, Header::getValue)));
		} catch (URISyntaxException | IOException e) {
			throw new IllegalStateException(e);
		}
	}
	
	protected abstract org.apache.http.client.fluent.Request createRequest() throws URISyntaxException;
}
